/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "hallcurrent5.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define HALLCURRENT5_ADC_RESOLUTION_10bit  1023.0    
#define HALLCURRENT5_ADC_RESOLUTION_12bit  4095.0

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void hallcurrent5_cfg_setup ( hallcurrent5_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->an_pin = HAL_PIN_NC;
    
    // Additional gpio pins

    cfg->int_pin = HAL_PIN_NC;

    cfg->channel      = ADC_CHANNEL_NC;
    cfg->resolution   = ADC_RESOLUTION_DEFAULT;
    cfg->vref         = ADC_VREF_DEFAULT;
    
    cfg->device_resolution = 0; 
    cfg->init_volt = 0;
}

HALLCURRENT5_RETVAL hallcurrent5_init ( hallcurrent5_t *ctx, hallcurrent5_cfg_t *cfg )
{
     analog_in_config_t adc_cfg;

    analog_in_configure_default( &adc_cfg );
    adc_cfg.pin        = cfg->an_pin;
    adc_cfg.channel    = cfg->channel;
    adc_cfg.resolution = cfg->resolution;
    adc_cfg.vref       = cfg->vref;
 
    ctx->dev_resolution = cfg->device_resolution;
    ctx->dev_init_volt = cfg->init_volt;
    
    if (  analog_in_open( &ctx->adc, &adc_cfg ) != ADC_SUCCESS )
    {
        return HALLCURRENT5_INIT_ERROR;
    }

    // Input pins

    digital_in_init( &ctx->int_pin, cfg->int_pin );

    ctx->dev_resolution = 1023.0; 
    ctx->dev_init_volt = 316;

    return HALLCURRENT5_OK;

}

hallcurrent5_data_t hallcurrent5_generic_read ( hallcurrent5_t *ctx )
{
    hallcurrent5_data_t rx_data;

    rx_data =  analog_in_read( &ctx->adc );
    
    return rx_data;
}

void hallcurrent5_set_adc_resolution ( hallcurrent5_t *ctx, float adc_res )
{
    ctx->dev_resolution = adc_res;
}

float hallcurrent5_get_adc_voltage ( hallcurrent5_t *ctx, uint16_t adc_value )
{
    float voltage;

    voltage = (float)( adc_value * (float)( 3300 / ctx->dev_resolution ) );
    return voltage;
}

float hallcurrent5_get_current ( hallcurrent5_t *ctx, uint16_t adc_value )
{
    float voltage;
    float current;
    
    voltage = (float)( adc_value * (float)( 3300 / ctx->dev_resolution ) );
    current = ( ( voltage - ctx->dev_init_volt - 12 ) / 66.0 ) * 1000.0;
    
    return current;
}

uint8_t hallcurrent5_get_fault ( hallcurrent5_t *ctx )
{
    return digital_in_read( &ctx->int_pin );
}

void hallcurrent5_set_init_voltage ( hallcurrent5_t *ctx, uint16_t voltage )
{
    ctx->dev_init_volt = voltage;
}

// ------------------------------------------------------------------------- END

