/*!
 * \file 
 * \brief HallCurrent5 Click example
 * 
 * # Description
 * The demo application reads ADC value and current value.
 *
 * The demo application is composed of two sections :
 * 
 * ## Application Init 
 * Initialization device and ADC Init.
 * 
 * ## Application Task  
 * Reads current value in mA and this data logs to USBUART every 500ms.
 * 
 * *note:* 
 * Before the start of the program you have to set the starting voltage on AN pin.
 * First, measure the voltage in mV on AN pin, when electronic load isn't connected to the Click board, 
 * and pass that value as an input parameter of the voltage initialization function.
 * 
 * \author MikroE Team
 *
 */
// ------------------------------------------------------------------- INCLUDES

#include "board.h"
#include "log.h"
#include "hallcurrent5.h"

// ------------------------------------------------------------------ VARIABLES

static hallcurrent5_t hallcurrent5;
static log_t logger;

// ------------------------------------------------------ APPLICATION FUNCTIONS

void application_init ( void )
{
    log_cfg_t log_cfg;
    hallcurrent5_cfg_t cfg;

    //  Logger initialization.

    log_cfg.level = LOG_LEVEL_DEBUG;
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, "---- Application Init ----" );

    //  Click initialization.

    hallcurrent5_cfg_setup( &cfg );
    HALLCURRENT5_MAP_MIKROBUS( cfg, MIKROBUS_1 );
    hallcurrent5_init( &hallcurrent5, &cfg );

    // CLICK INIT
    hallcurrent5_set_adc_resolution( &hallcurrent5, HALLCURRENT5_ADC_RESOLUTION_12bit );
    hallcurrent5_set_init_voltage( &hallcurrent5, 316 );
    Delay_100ms();
}

void application_task ( void )
{
    hallcurrent5_data_t tmp;
    int16_t adc_value;
    uint32_t sum_adc = 0;
    uint8_t cnt;
    float current;
  
    //  Task implementation.
    
    tmp = hallcurrent5_generic_read( &hallcurrent5 );
    log_printf( &logger, "** ADC value : [DEC]- %d, [HEX]- 0x%x \r\n", tmp, tmp );
    Delay_ms( 1000 );

    sum_adc = 0;
    for ( cnt = 0; cnt < 20; cnt++ )
    {
        sum_adc += hallcurrent5_generic_read( &hallcurrent5 );
    }
    adc_value = ( sum_adc / 20 );

    current = hallcurrent5_get_current( &hallcurrent5, adc_value/1.8 );

    if ( current < 0 ) 
    {
        current = 0;
    }

    log_printf( &logger, " ADC value: %d \r\n", adc_value);
    log_printf( &logger, " Current : \r\n", adc_value);

    Delay_ms( 500 );
}

void main ( void )
{
    application_init( );

    for ( ; ; )
    {
        application_task( );
    }
}


// ------------------------------------------------------------------------ END
